import {
	btn,
	checkbox,
	checkboxcolumn,
	CheckboxField,
	column,
	comp,
	Component,
	datasourcestore,
	DefaultEntity,
	EntityID,
	h3,
	h4,
	hr,
	menu,
	t,
	Table,
	table,
	tbar,
	Window,
	ComponentState, h1
} from "@intermesh/goui";
import {jmapds} from "@intermesh/groupoffice-core";

export class ProjectTasksDetail extends Component {
	private projectId?: EntityID;
	private showCompleteCB!: CheckboxField;
	private showComplete = true;

	constructor() {
		super();
		this.cls = "card vbox";
		this.stateId = "project-tasks-detail";
	}

	protected buildState(): ComponentState {
		return {showComplete: this.showComplete};
	}

	protected restoreState(state: ComponentState): void {
		this.showComplete = state.showComplete;
	}

	private createTable(project: DefaultEntity, mileStone: DefaultEntity | undefined) {
		const mileStoneId = (mileStone ? mileStone.id : null), items = [];


		if (mileStone) {
			items.push(tbar({}, h4(mileStone.name), "->", // btn({
				// 	icon: "expand_less",
				// 	handler: (btn) => {
				// 		const next = btn.parent!.nextSibling() as Component;
				// 		next.hidden = !next.hidden;
				// 		btn.icon = next.hidden ? "expand_more" : "expand_less";
				// 	}
				// })
			));
		}

		items.push(comp({cls: "scroll", flex: 1}, table({
				hidden: true, // cls: "fit",
				fitParent: true, //groupBy: "milestone",
				sortableGroup: "milestone-tasks", dropBetween: false, rowSelectionConfig: {
					multiSelect: true
				}, dropOn: true, draggable: true, listeners: {

					drop: async (toComponent, toIndex, fromIndex, droppedOn, fromComp, dragDataSet) => {
						const fromTable = (fromComp as Table), selectedRowIndexes = dragDataSet.selectedRowIndexes as number[];

						// map to records
						const fromRecords = selectedRowIndexes.map(index => fromTable.store.get(index)!);
						fromTable.rowSelection!.clear();

						this.mask();
						try {
							// send updates to DataSource
							await Promise.all(fromRecords.map(fromRecord => {
								jmapds("Task").update(fromRecord.id, {mileStoneId: toComponent.store.getFilter("project").mileStoneId})
							}));
						} catch (e) {
							void Window.error(e);
						} finally {
							this.unmask();
						}
					},

					render: (table) => {
						table.store.on("datachanged", () => {

							table.hidden = table.store.count() == 0;
						})

						void table.store.load();


					},

					rowdblclick: (list, storeIndex, row, ev) => {
						const win = new go.links.LinkDetailWindow({
							entity: "Task"
						});
						win.load(list.store.get(storeIndex)!.id);
					},
				}, columns: [checkboxcolumn({
					id: "complete", listeners: {
						change: async (col, field, value, record, storeIndex) => {
							this.mask();
							try {
								await jmapds("Task").update(record.id, {progress: (value ? 'completed' : 'needs-action')})
							} catch (e) {
								void Window.error(e);
							} finally {
								this.unmask();
							}
						}
					}
				}), column({
					resizable: true, id: "title", header: t("Title")
				}),

					column({
						width: 200, resizable: true, id: "responsible", property: "responsible/name", header: t("Assigned To")
					}),

					column({
						sticky: true, width: 32, id: "btn", renderer: (columnValue: any, record, td, table, rowIndex) => {

							return btn({
								icon: "more_vert", menu: menu({}, btn({
									icon: "edit", text: t("Edit"), handler: async (btn) => {
										const dlg = new go.modules.community.tasks.TaskDialog({
											redirectOnSave: false
										});

										const task = table.store.get(rowIndex)!;

										dlg.load(task.id);
										dlg.show();
									}
								}), hr(), btn({
									icon: "delete", text: t("Delete"), handler: async (btn) => {
										const task = table.store.get(rowIndex)!;
										jmapds("Task").confirmDestroy([task.id]);
									}
								}))
							})
						}
					})],
				store: datasourcestore({
					dataSource: jmapds("Task"),
					filters: {
						tasklist: {taskListId: "any", complete: this.showCompleteCB.value ? undefined : false},
						project: {projectId: project.id, mileStoneId}
					},
					relations: {
						responsible: {
							dataSource: jmapds("Principal"), path: "responsibleUserId"
						},

					},
					buildRecord: async entity => {
						return Object.assign({complete: entity.progress == "completed"}, entity);
					}
				})
			})),

			tbar({}, "->", btn({
				cls: "", icon: "add", text: t("Add task"), handler: async () => {

					const project = (await jmapds("Project3").single(this.projectId!))!,
						projectBook = (await jmapds("ProjectBook").single(project.bookId))!;

					const dlg = new go.modules.community.tasks.TaskDialog({
						redirectOnSave: false
					});
					dlg.show();
					dlg.setValues({projectId: this.projectId, tasklistId: projectBook.tasklistId, mileStoneId: mileStoneId})
				}
			})));
		return items;

	}


	public async load(projectId: EntityID) {
		this.projectId = projectId;

		const project = (await jmapds("Project3").single(projectId))!;

		this.items.replace(
			tbar({}, h3(t("Tasks")), "->",
				// btn({
				// 	icon: "expand_less",
				// 	handler: (btn) => {
				// 		const next = btn.parent!.nextSibling() as Component;
				// 		next.hidden = !next.hidden;
				// 		btn.icon = next.hidden ? "expand_more" : "expand_less";
				// 	}
				// })

				this.showCompleteCB = checkbox({
					type: "switch",
					name: "showComplete",
					label: t("Show complete"),
					value: this.showComplete,
					listeners: {
						change: (cb, v) => {
							this.showComplete = v;

							this.saveState();
							void this.load(this.projectId!);
						}
					}
				})
			)
		)
		this.items.add(...this.createTable(project, undefined));

		if (project.milestones) {
			project.milestones.forEach((m: any) => {
				this.items.add(...this.createTable(project, m));
			})
		}

		this.items.add(tbar({}, "->", btn({
			icon: "add", text: t("Add milestone"), handler: async () => {

				const project = (await jmapds("Project3").single(this.projectId!))!;

				const name = await Window.prompt("", t("Name"), "", t("Add milestone"));

				if (!name) {
					return;
				}

				const milestones = project.milestones ?? [];

				milestones.push({name});

				await jmapds("Project3").update(this.projectId!, {milestones: milestones})

			}
		})));

	}
}